import psutil
import os
from .beacon import Beacon

class Service(Beacon):
    '''
    This beacon is monitoring Cpu stuff
    Captain Obvious strikes!
    '''
    #name = "Cpu"
    graphite_namespace = "cpu"
    expose = ['load_5', 'usage']

    def expose_load_5(self):
        return dict(
            graphite_namespace="load_5",
            value=str(os.getloadavg()[0])[:4]
        )

    def expose_load_10(self):
        return dict(
            graphite_namespace="load_10",
            value=str(os.getloadavg()[1])[:4]
        )

    def expose_load_15(self):
        return dict(
            graphite_namespace="load_15",
            value=str(os.getloadavg()[2])[:4]
        )

    def expose_usage(self):
        return dict(
            graphite_namespace="usage_percent",
            value=psutil.cpu_percent(interval=1)
        )


import psutil
from .beacon import Beacon

class Service(Beacon):
    '''
    This beacon is monitoring memory stuff
    Captain Obvious strikes, again!!
    '''
    #name = "Memory"
    graphite_namespace = "memory"
    expose = ["percent", "used", "available", "total"]

    def expose_percent(self):
        return dict(
            graphite_namespace="percent",
            value=psutil.virtual_memory().percent
        )

    def expose_used(self):
        return dict(
            graphite_namespace="used",
            value=self.b_to_mb(psutil.virtual_memory().used)
        )

    def expose_available(self):
        return dict(
            graphite_namespace="available",
            value=self.b_to_mb(
                psutil.virtual_memory().total - psutil.virtual_memory().used)
        )

    def expose_sw_percent(self):
        return dict(
            graphite_namespace="sw_percent",
            value=psutil.virtual_memory().percent
        )

    def expose_sw_used(self):
        return dict(
            graphite_namespace="sw_used",
            value=self.b_to_mb(psutil.virtual_memory().used)
        )

    def expose_sw_available(self):
        return dict(
            graphite_namespace="sw_available",
            value=self.b_to_mb(
                psutil.virtual_memory().total - psutil.virtual_memory().used)
        )

    def b_to_mb(self, bytes):
        # Convert b to Mb
        return bytes/1000000



import psutil
import os
from .beacon import Beacon

class Service(Beacon):
    '''
    Get usage disk information and other stuff
    '''
    graphite_namespace = "disk"
    expose = ['percent_usage']

    def expose_percent_usage(self):
        data = []
        namespace = 'percent_usage'
        for part in psutil.disk_partitions():
            partitions = {}
            temp_device = self.format(part.device, namespace)
            partitions[temp_device] = psutil.disk_usage(
                part.mountpoint
                ).percent
            data.append(partitions)

        return data

    def format(self, device, sufix):
        name = device.split('/')[-1]
        return '{}_{}'.format(name, sufix)



class Beacon(object):
    graphite_namespace = ''
    expose = []

    def join_namespaces(self, namespace):
        return u'{}.{}'.format(
            self.graphite_namespace,
            namespace
        )

    def run(self):
        for service in self.expose:
            method = getattr(self, 'expose_{}'.format(service), None)
            if method:  # Handle unexistant method!
                data = method()
                if type(data) is dict:
                    namespace = self.join_namespaces(
                        data['graphite_namespace']
                        )
                    data['graphite_namespace'] = namespace
                    yield data

                elif type(data) is list:
                    for inner_data in data:
                        new_data = {}
                        new_data['graphite_namespace'] = str(
                            inner_data.keys()).strip('[]\''
                            )
                        new_data['value'] = str(
                            inner_data.values()
                            ).strip('[]')
                        namespace = self.join_namespaces(
                            new_data['graphite_namespace']
                            )
                        new_data['graphite_namespace'] = namespace
                        yield new_data


import os
from setuptools import setup, find_packages
import re
import sys

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...

def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search(
        "^__version__ = ['\"]([^'\"]+)['\"]",
        init_py, re.MULTILINE).group(1)

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

package = 'pylon'
version = get_version(package)

if sys.argv[-1] == 'publish':
    os.system("python setup.py sdist upload")
    args = {'version': version}
    print("You probably want to also tag the version now:")
    print("  git tag -a %(version)s -m 'version %(version)s'" % args)
    print("  git push --tags")
    sys.exit()

setup(
    name = "pyl0n",
    version = version,
    author = "Juan Manuel Parrilla Madrid",
    author_email = "padajuan@gmail.com",
    description = ("Use psutils to send data to Graphite Backend"),
    license = "GPLv2",
    keywords = "psutil pylon monitoring graphite",
    url = "https://github.com/padajuan/pylon",
    include_package_data=True,
    packages=find_packages(),
    install_requires=open('requirements.txt').read().split('\n'),
    long_description=read('README.md'),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Operating System :: POSIX :: Linux',
        'Topic :: System :: Monitoring',
        'Topic :: System :: Operating System',
    ],
)


import ConfigParser
import sys
import logging
import time
import os
from os.path import realpath
from os.path import dirname
import socket
from importlib import import_module
from beacons import beacon
import simpledaemon

class BeaconHandler(object):
    '''
    This Class will manage all beacons, register all beacons that
    it is in config file and call all of them in a row.
    (not async operations yet)
    '''
    graphite_namespace = None
    beacons = {}

    def __init__(self):
        self.get_config()
        self.logger()
        self.conn = self.setup_graphite()

    def get_config(self):
        # Configuration catcher
        file_path = dirname(realpath(__file__))
        config = ConfigParser.RawConfigParser()
        try:
            logging.info('Loading configuration')
            config.read(file_path + '/probe.ini')
        except Exception as err:
            logging.critical('Error reading config file')
            raise Exception('Error reading config file: %s' % err)

        logging.info('Configuration loaded correctly')
        self.server = config.get('Graphite', 'server')
        self.port = config.get('Graphite', 'port')
        self.delay = config.get('Graphite', 'delay')
        self.domain = config.get('Graphite', 'domain')
        self.initiative = config.get('Graphite', 'initiative')
        self.hostname = config.get('Graphite', 'hostname')
        if not self.hostname:
            self.hostname = socket.gethostname()

        self.loaded_beacons = config.get('Graphite', 'beacons')
        self.graphite_log = config.get('Graphite', 'logfile')
        self.logLevel = config.get('Graphite', 'loglevel')
        if not self.logLevel:
            self.logLevel = info

        self.daemonPid = config.get('Graphite', 'pidfile')

    def logger(self):
        # Logger function
        log_file = self.graphite_log
        level = self.logLevel
        logging.getLogger('').handlers = []
        if not os.path.exists(log_file):
            open(log_file, 'a').close()

        logging.basicConfig(
            filename=log_file,
            format='%(asctime)-15s %(name)-5s %(levelname)-8s %(message)s',
            level=logging.INFO
            )

        logging.info('Pylon has been Activated')


    def register(self, new_beacon):
        # Register a Beacon, like a plugin activation
        class_name = new_beacon().__class__
        if class_name not in self.beacons:
            if isinstance(new_beacon(), beacon.Beacon):
                self.beacons[class_name] = new_beacon

    def setup_graphite(self):
        # Connect with graphite server
        try:
            sock = socket.socket()
            sock.connect((self.server, int(self.port)))
        except Exception as error:
            logging.error("Couldn't connect to {} on port {}".format(
                self.server, self.port))
            logging.error(error)
            sys.exit(1)

        return sock

    def beacon_loader(self, modules):
        # Dinamyc load of beacons
        for module in modules.split(","):
            mod = import_module('beacons.{}'.format(module.strip()))
            self.register(mod.Service)

    def send_to_graphite(self, namespace, value, timestamp):
        # Log all data catched and send to graphite
        logging.debug(u'{} {} {}'.format(
            namespace, 
            value,
            timestamp,
            )
        )
        try:
            self.conn.sendall ('{} {} {} \n'.format(
                namespace,
                value,
                timestamp
                )
            )

            # print '{} {} {} \n'.format(
            #     namespace,
            #     value,
            #     timestamp
            #     )

        except Exception as err:
            logging.error("Error sending information to graphite: %s" % err)
            raise Exception('Error sending information to graphite: %s' % err)


    def run(self):
        # Get data from beacon to notify it to graphite
        for beacon_name,beacon_class in self.beacons.iteritems():
            beacon = beacon_class()
            for data in beacon.run():
                timestamp = int(time.time())
                path = u'{}.{}.{}.{}'.format(
                    self.domain,
                    self.initiative,
                    self.hostname,
                    data['graphite_namespace']
                    )
                self.send_to_graphite(
                    path,
                    data['value'],
                    timestamp,
                )

    def close_conn(self):
        self.conn.close()

class Nexus(simpledaemon.Daemon):
    '''
    This Class is a layer over the BeaconHandler to manage software as a daemons
    '''
    file_path = dirname(realpath(__file__))
    default_conf = file_path + '/probe.ini'
    section = 'Graphite'

    def run(self):
        handler = BeaconHandler()
        while True:
            handler.beacon_loader(handler.loaded_beacons)
            handler.run()
            time.sleep(int(handler.delay))

        handler.close_conn()

if __name__ == "__main__":
    Nexus().main()
    
